#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import os.path

def chop(line):
    if line.endswith('\r\n'):
        return line[:-2]
    elif line.endswith('\n'):
        return line[:-1]
    elif line.endsiwth('\r'):
        return line[:-1]

def backup(filepath):
    backupfilepath = filepath + '.bak'
    if os.path.exists(backupfilepath):
        count = 0
        while os.path.exists(backupfilepath):
            count += 1
            backupfilepath = filepath + (".bak%d" % count)
    os.rename(filepath, backupfilepath)

__MARKER = '#table generated by xedstrtsv.py'
__TEMPFILENAME = 'xedstrtsv_temp_file.tmp'

def generatetable(asciiexternalizedfilename, nativeexternalizedfilename, out):
    def keycmp(key1, key2):
        k1 = key1.lower()
        k2 = key2.lower()
        if k1 < k2:
            return -1
        elif k1 == k2:
            return 0
        return 1
    
    af = open(asciiexternalizedfilename, "r")
    alines = [ chop(line) for line in af ]
    alines.sort(cmp=keycmp)
    af.close()
    
    r = os.system("native2ascii -reverse %s %s" % (nativeexternalizedfilename, __TEMPFILENAME))
    assert r == 0
    
    nf = open(__TEMPFILENAME, "r")
    nlines = [ chop(line) for line in nf ]
    nlines.sort(cmp=keycmp)
    nf.close()
    
    def splitbyequal(line):
        assert line.count('\t') == 0
        pos = line.index('=')
        if pos == len(line) - 1:
            key, value = line[0:pos], ""
        else:
            key, value = line[0:pos], line[pos + 1:]
        return key, value
    
    print >> out, __MARKER
    print >> out, "key\t%s\t%s" % (asciiexternalizedfilename, nativeexternalizedfilename)
    alc, nlc = len(alines), len(nlines)
    ai, ni = 0, 0
    while ai < alc and ni < nlc:
        k1, v1 = splitbyequal(alines[ai])
        k2, v2 = splitbyequal(nlines[ni])
        c = keycmp(k1, k2)
        if c == 0:
            print >> out, "%s\t%s\t%s" % (k1, v1, v2)
            ai += 1
            ni += 1
        elif c < 0:
            print >> out, "%s\t%s\t" % (k1, v1)
            ai += 1
        else:
            print >> out, "%s\t\t%s" % (k2, v2)
            ni += 1
    while ai < alc:
        k1, v1 = splitbyequal(alines[ai])
        print >> out, "%s\t%s\t" % (k1, v1)
        ai += 1
    while ni < nlc:
        k2, v2 = splitbyequal(nlines[ni])
        print >> out, "%s\t\t%s" % (k2, v2)
        ni += 1
    
def splittable(tablefile):
    lines = [ chop(line) for line in tablefile ]
    assert lines[0].startswith(__MARKER)
    fields = lines[1].split('\t')
    assert len(fields) == 3
    assert fields[0] == 'key'
    aoutfilename, noutfilename = fields[1], fields[2]
    
    if os.path.exists(aoutfilename):
        backup(aoutfilename)
    if os.path.exists(noutfilename):
        backup(noutfilename)
    
    f1 = open(aoutfilename, 'w')
    f2 = open(__TEMPFILENAME, 'w')
    for line in lines[2:]:
        fields = line.split('\t')
        key = fields[0]
        while len(fields) < 3:
            fields.append('')
        assert len(fields) == 3
        v1, v2 = fields[1], fields[2]
        print >> f1, "%s=%s" % (key, v1)
        print >> f2, "%s=%s" % (key, v2)
            
    f1.close()
    f2.close()
    
    r = os.system("native2ascii %s %s" % (__TEMPFILENAME, noutfilename))
    assert r == 0
        
if __name__ == '__main__':
    import sys
    import getopt

    usage = """
usage 1: xedstrtsv [-o table.tsv] messages.properties messages_ja.properties
  make a table containing externalized strings of two files.
usage 2: xedstrtsv [-i table.tsv]
  split the table and generate two files of externalized strings.
"""[1:-1]

    outfilename = None
    inpfilename = None
    opts, args = getopt.getopt(sys.argv[1:], 'hi:o:')
    for key, value in opts:
        if key == '-h':
            print usage
            sys.exit(1)
        elif key == '-o':
            outfilename = value
        elif key == '-i':
            inpfilename = value
    
    if len(args) == 2:
        assert inpfilename is None
        if outfilename:
            if os.path.exists(outfilename):
                backup(outfilename)
            out = open(outfilename, 'w')
            generatetable(args[0], args[1], out)
            out.close()
        else:
            generatetable(args[0], args[1], sys.stdout)
    elif len(args) == 0:
        assert outfilename is None
        if inpfilename:
            inp = open(inpfilename, 'r')
            splittable(inp)
            inp.close()
        else:
            splittable(sys.stdin)
    else:
        assert False

            
    
    
        